@@ -97,11 +97,7 @@ notification_plugin_configure (XfcePanelPlugin      *plugin,
   gtk_container_add_with_properties (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 						                         grid, "expand", TRUE, "fill", TRUE, NULL);
   label = gtk_label_new (_("Number of notifications to show"));
-#if GTK_CHECK_VERSION (3, 16, 0)
   gtk_label_set_xalign (GTK_LABEL (label), 0);
-#else
-  gtk_widget_set_halign (label, GTK_ALIGN_START);
-#endif
   gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (label), 0, 0, 1, 1);
   spin = gtk_spin_button_new (adjustment, 1.0, 0);
   gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (spin), 1, 0, 1, 1);
@@ -109,11 +105,7 @@ notification_plugin_configure (XfcePanelPlugin      *plugin,
                           G_OBJECT (spin), "value");
 
   label = gtk_label_new (_("Only show notifications from today"));
-#if GTK_CHECK_VERSION (3, 16, 0)
   gtk_label_set_xalign (GTK_LABEL (label), 0);
-#else
-  gtk_widget_set_halign (label, GTK_ALIGN_START);
-#endif
   gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (label), 0, 1, 1, 1);
   check = gtk_switch_new ();
   gtk_widget_set_halign (check, GTK_ALIGN_END);
@@ -122,11 +114,7 @@ notification_plugin_configure (XfcePanelPlugin      *plugin,
                           G_OBJECT (check), "active");
 
   label = gtk_label_new (_("Hide 'Clear log' confirmation dialog"));
-#if GTK_CHECK_VERSION (3, 16, 0)
   gtk_label_set_xalign (GTK_LABEL (label), 0);
-#else
-  gtk_widget_set_halign (label, GTK_ALIGN_START);
-#endif
   gtk_grid_attach (GTK_GRID (grid), GTK_WIDGET (label), 0, 2, 1, 1);
   check = gtk_switch_new ();
   gtk_widget_set_halign (check, GTK_ALIGN_END);
@@ -134,6 +122,15 @@ notification_plugin_configure (XfcePanelPlugin      *plugin,
   xfconf_g_property_bind (notification_plugin->channel, SETTING_HIDE_CLEAR_PROMPT, G_TYPE_BOOLEAN,
                           G_OBJECT (check), "active");
 
+  label = gtk_label_new(_("Hide panel button when no unread notifications"));
+  gtk_label_set_xalign(GTK_LABEL(label), 0);
+  gtk_grid_attach(GTK_GRID(grid), label, 0, 3, 1, 1);
+  check = gtk_switch_new();
+  gtk_widget_set_halign(check, GTK_ALIGN_END);
+  gtk_grid_attach(GTK_GRID(grid), check, 1, 3, 1, 1);
+  xfconf_g_property_bind(notification_plugin->channel, SETTING_HIDE_ON_READ, G_TYPE_BOOLEAN,
+                         G_OBJECT(check), "active");
+
   /* link the dialog to the plugin, so we can destroy it when the plugin
    * is closed, but the dialog is still open */
   g_object_set_data (G_OBJECT (plugin), "dialog", dialog);
