@@ -34,6 +34,7 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4panel/libxfce4panel.h>
 
+#include <common/xfce-notify-common.h>
 #include <common/xfce-notify-log.h>
 
 #include "notification-plugin.h"
@@ -138,11 +139,7 @@ notification_plugin_menu_populate (NotificationPlugin *notification_plugin)
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   label = gtk_label_new (NULL);
   gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), _("<b>_Do not disturb</b>"));
-#if GTK_CHECK_VERSION (3, 16, 0)
   gtk_label_set_xalign (GTK_LABEL (label), 0);
-#else
-  gtk_widget_set_halign (label, GTK_ALIGN_START);
-#endif
   notification_plugin->do_not_disturb_switch = gtk_switch_new ();
   gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (box), notification_plugin->do_not_disturb_switch, FALSE, FALSE, 0);
@@ -239,31 +236,20 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       g_free (tmp);
       summary = gtk_label_new (NULL);
       gtk_label_set_markup (GTK_LABEL (summary), markup);
-#if GTK_CHECK_VERSION (3, 16, 0)
       gtk_label_set_xalign (GTK_LABEL (summary), 0);
-#else
-      gtk_widget_set_halign (summary, GTK_ALIGN_START);
-#endif
       gtk_label_set_ellipsize (GTK_LABEL (summary), PANGO_ELLIPSIZE_END);
       gtk_label_set_max_width_chars (GTK_LABEL (summary), 40);
       g_free (markup);
 
       tmp = g_key_file_get_string (notify_log, group, "body", NULL);
       body = gtk_label_new (NULL);
-      gtk_label_set_markup (GTK_LABEL (body), tmp);
-      if (g_strcmp0 (gtk_label_get_text(GTK_LABEL (body)), "") == 0) {
-        gchar *tmp1;
-
-        tmp1 = g_markup_escape_text (tmp, -1);
-        gtk_label_set_text (GTK_LABEL (body), tmp1);
-        g_free (tmp1);
+      if (xfce_notify_is_markup_valid(tmp)) {
+          gtk_label_set_markup (GTK_LABEL (body), tmp);
+      } else {
+          gtk_label_set_text(GTK_LABEL(body), tmp);
       }
       g_free (tmp);
-#if GTK_CHECK_VERSION (3, 16, 0)
       gtk_label_set_xalign (GTK_LABEL (body), 0);
-#else
-      gtk_widget_set_halign (body, GTK_ALIGN_START);
-#endif
       gtk_label_set_ellipsize (GTK_LABEL (body), PANGO_ELLIPSIZE_END);
       gtk_label_set_max_width_chars (GTK_LABEL (body), 40);
 
