@@ -29,6 +29,8 @@
 #include <string.h>
 #endif
 
+#include <gio/gio.h>
+#include <gio/gdesktopappinfo.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkx.h>
 
@@ -36,12 +38,12 @@
 #include <libxfce4ui/libxfce4ui.h>
 #include <libnotify/notify.h>
 
+#include <common/xfce-notify-common.h>
 #include <common/xfce-notify-log.h>
 
 #include "xfce4-notifyd-config.ui.h"
+#include "xfce4-notifyd-config-known-app.ui.h"
 
-#define KNOWN_APPLICATIONS_PROP       "/applications/known_applications"
-#define MUTED_APPLICATIONS_PROP       "/applications/muted_applications"
 #define LOG_DISPLAY_LIMIT             100
 
 typedef struct
@@ -60,6 +62,30 @@ typedef struct
     GtkWidget  *do_slideout_label;
 } NotificationSlideoutWidgets;
 
+typedef struct
+{
+    const gchar *app_id;
+    gchar *app_name;
+    gint count;
+} LogAppCount;
+
+typedef struct
+{
+    XfconfChannel *channel;
+    gchar *known_application;
+    gchar *known_application_name;
+
+    GtkWidget *app_label;
+    GtkWidget *mute_switch;
+    GtkWidget *allow_criticals;
+    GtkWidget *include_in_log;
+} KnownApplicationSignalData;
+
+static void handle_app_switch_property_changed(XfconfChannel *channel,
+                                               const gchar *property_name,
+                                               const GValue *value,
+                                               KnownApplicationSignalData *signal_data);
+
 static void
 xfce_notifyd_config_show_notification_callback(NotifyNotification *notification,
                                                const char         *action,
@@ -77,7 +103,7 @@ xfce_notifyd_config_show_notification_preview(GtkWindow *parent_window)
     notification =
         notify_notification_new(_("Notification Preview"),
                                 _("This is what notifications will look like"),
-                                "xfce4-notifyd");
+                                "org.xfce.notification");
 
     notify_notification_add_action(notification,
                                    "button",
@@ -281,91 +307,170 @@ display_header_func (GtkListBoxRow *row,
     }
 }
 
-static void
-xfce4_notifyd_mute_application (GtkListBox *known_applications_listbox,
-                                GtkListBoxRow *selected_application_row,
-                                XfconfChannel *channel)
+static gboolean
+xfce_g_strv_remove(gchar **strv,
+                   const gchar *to_remove)
 {
+    gint i;
 
-    GtkWidget *application_box;
-    GtkWidget *application_label;
-    GtkWidget *mute_switch;
-    gboolean muted;
-    GPtrArray *muted_applications;
-    GValue *val;
-    guint i;
-    const gchar *application_name;
-    gchar *new_app_name;
-
-    muted_applications = xfconf_channel_get_arrayv (channel, MUTED_APPLICATIONS_PROP);
-    if (muted_applications == NULL)
-        muted_applications = g_ptr_array_new ();
-
-    application_box = GTK_WIDGET (g_list_nth_data (gtk_container_get_children (GTK_CONTAINER (selected_application_row)), 0));
-    application_label = GTK_WIDGET (g_list_nth_data (gtk_container_get_children (GTK_CONTAINER (application_box)), 1));
-    application_name = gtk_label_get_text (GTK_LABEL(application_label));
-    mute_switch = GTK_WIDGET (g_list_nth_data (gtk_container_get_children (GTK_CONTAINER (application_box)), 2));
-    muted = gtk_switch_get_active (GTK_SWITCH (mute_switch));
-
-    val = g_new0 (GValue, 1);
-    g_value_init (val, G_TYPE_STRING);
-    new_app_name = g_strdup (application_name);
-    g_value_take_string (val, new_app_name);
-    if (muted == TRUE && muted_applications != NULL) {
-        for (i = 0; i < muted_applications->len; i++) {
-            GValue *muted_application;
-            muted_application = g_ptr_array_index (muted_applications, i);
-            if (g_str_match_string (g_value_get_string (val), g_value_get_string (muted_application), FALSE) == TRUE) {
-                if (!g_ptr_array_remove_index (muted_applications, i))
-                    g_warning ("Could not remove %s from the list of muted applications. %i", new_app_name, i);
-                break;
-            }
-
+    for (i = 0; strv[i] != NULL; ++i) {
+        if (strcmp(strv[i], to_remove) == 0) {
+            break;
         }
-
     }
-    else
-        g_ptr_array_add (muted_applications, val);
 
-    if (!xfconf_channel_set_arrayv (channel, MUTED_APPLICATIONS_PROP, muted_applications))
-        g_warning ("Could not add %s to the list of muted applications.", new_app_name);
+    if (G_LIKELY(strv[i] != NULL)) {
+        gint remaining = g_strv_length(&strv[i+1]);
 
-    xfconf_array_free (muted_applications);
+        g_free(strv[i]);
+
+        if (remaining > 0) {
+            memmove(&strv[i], &strv[i+1], remaining * sizeof(gchar *));
+        }
+        strv[i+remaining] = NULL;
+
+        return TRUE;
+    } else {
+        return FALSE;
+    }
 }
 
 static void
-xfce4_notifyd_row_activated (GtkListBox *known_applications_listbox,
-                             GtkListBoxRow *selected_application_row,
-                             gpointer user_data)
+xfce_add_to_string_list_property(XfconfChannel *channel,
+                                 const gchar *property_name,
+                                 gchar **strv,
+                                 const gchar *to_add)
 {
-    GtkWidget *application_box;
-    GtkWidget *mute_switch;
-    gboolean muted;
-
-    application_box = GTK_WIDGET (g_list_nth_data (gtk_container_get_children (GTK_CONTAINER (selected_application_row)), 0));
-    mute_switch = GTK_WIDGET (g_list_nth_data (gtk_container_get_children (GTK_CONTAINER (application_box)), 2));
-    muted = !gtk_switch_get_active (GTK_SWITCH (mute_switch));
-    gtk_switch_set_active (GTK_SWITCH (mute_switch), muted);
+    if (strv == NULL || !g_strv_contains((const gchar *const *)strv, to_add)) {
+        gint len = strv == NULL ? 0 : g_strv_length(strv);
+        gchar **new_strv = g_new(gchar *, len + 2);
+        if (len > 0) {
+            memcpy(new_strv, strv, len * sizeof(gchar *));
+        }
+        new_strv[len] = (gchar *)to_add;
+        new_strv[len + 1] = NULL;
+        xfconf_channel_set_string_list(channel, property_name, (const gchar *const *)new_strv);
+        g_free(new_strv);
+    }
 }
 
 static void
-xfce4_notifyd_switch_activated (GtkSwitch *mute_switch,
-                                gboolean state,
-                                gpointer user_data)
+xfce_remove_from_string_list_property(XfconfChannel *channel,
+                                      const gchar *property_name,
+                                      gchar **strv,
+                                      const gchar *to_remove)
 {
-    XfconfChannel *channel = user_data;
-    GtkWidget *row_box;
-    GtkWidget *selected_application_row;
-    GtkWidget *known_applications_listbox;
+    if (strv != NULL) {
+        if (xfce_g_strv_remove(strv, to_remove)) {
+            if (strv[0] == NULL) {
+                xfconf_channel_reset_property(channel, property_name, FALSE);
+            } else {
+                xfconf_channel_set_string_list(channel, property_name, (const gchar *const *)strv);
+            }
+        }
+    }
+}
 
-    g_return_if_fail (XFCONF_IS_CHANNEL (channel));
+static void
+handle_app_switch_widget_toggled(KnownApplicationSignalData *signal_data,
+                                 const gchar *property_name,
+                                 gboolean add_to_list)
+{
+    gchar **strs = xfconf_channel_get_string_list(signal_data->channel, property_name);
 
-    row_box = gtk_widget_get_parent (GTK_WIDGET (mute_switch));
-    selected_application_row = gtk_widget_get_parent (GTK_WIDGET (row_box));
-    known_applications_listbox = gtk_widget_get_parent (GTK_WIDGET (selected_application_row));
-    xfce4_notifyd_mute_application (GTK_LIST_BOX (known_applications_listbox),
-                                    GTK_LIST_BOX_ROW (selected_application_row),
-                                    channel);
+    g_signal_handlers_block_by_func(signal_data->channel,
+                                    handle_app_switch_property_changed,
+                                    signal_data);
+    if (add_to_list) {
+        xfce_add_to_string_list_property(signal_data->channel, property_name, strs, signal_data->known_application);
+    } else {
+        xfce_remove_from_string_list_property(signal_data->channel, property_name, strs, signal_data->known_application);
+    }
+    g_signal_handlers_unblock_by_func(signal_data->channel,
+                                      handle_app_switch_property_changed,
+                                      signal_data);
+
+
+    g_strfreev(strs);
+}
+
+static void
+xfce4_notifyd_mute_switch_activated(GtkSwitch *sw,
+                                    gboolean state,
+                                    KnownApplicationSignalData *signal_data)
+{
+    handle_app_switch_widget_toggled(signal_data, MUTED_APPLICATIONS_PROP, state);
+    gtk_widget_set_sensitive(signal_data->app_label, !state);
+}
+
+static void
+xfce4_notifyd_allow_criticals_switch_activated(GtkSwitch *sw,
+                                               gboolean state,
+                                               KnownApplicationSignalData *signal_data)
+{
+    handle_app_switch_widget_toggled(signal_data, DENIED_CRITICAL_NOTIFICATIONS_PROP, !state);
+}
+
+static void
+xfce4_notifyd_include_in_log_switch_activated(GtkSwitch *sw,
+                                              gboolean state,
+                                              KnownApplicationSignalData *signal_data)
+{
+    handle_app_switch_widget_toggled(signal_data, EXCLUDED_FROM_LOG_APPLICATIONS_PROP, !state);
+}
+
+static void
+handle_app_switch_property_changed(XfconfChannel *channel,
+                                   const gchar *property_name,
+                                   const GValue *value,
+                                   KnownApplicationSignalData *signal_data)
+{
+    gboolean found = FALSE;
+    GtkWidget *switch_to_set;
+    GtkWidget *label_to_sensitize = NULL;
+    gpointer callback_to_block;
+    gboolean reverse_sense;
+
+    if (g_strcmp0(property_name, MUTED_APPLICATIONS_PROP) == 0) {
+        switch_to_set = signal_data->mute_switch;
+        label_to_sensitize = signal_data->app_label;
+        callback_to_block = xfce4_notifyd_mute_switch_activated;
+        reverse_sense = FALSE;
+    } else if (g_strcmp0(property_name, DENIED_CRITICAL_NOTIFICATIONS_PROP) == 0) {
+        switch_to_set = signal_data->allow_criticals;
+        callback_to_block = xfce4_notifyd_allow_criticals_switch_activated;
+        reverse_sense = TRUE;
+    } else if (g_strcmp0(property_name, EXCLUDED_FROM_LOG_APPLICATIONS_PROP) == 0) {
+        switch_to_set = signal_data->include_in_log;
+        callback_to_block = xfce4_notifyd_include_in_log_switch_activated;
+        reverse_sense = TRUE;
+    } else {
+        return;
+    }
+
+    if (G_VALUE_HOLDS(value, G_TYPE_PTR_ARRAY)) {
+        GPtrArray *strs = g_value_get_boxed(value);
+        for (guint i = 0; i < strs->len; ++i) {
+            const GValue *str = g_ptr_array_index(strs, i);
+            if (G_VALUE_HOLDS_STRING(str)) {
+                if (g_strcmp0(g_value_get_string(str), signal_data->known_application) == 0) {
+                    found = TRUE;
+                    break;
+                }
+            }
+        }
+    }
+
+    g_signal_handlers_block_by_func(signal_data->mute_switch,
+                                    callback_to_block,
+                                    signal_data);
+    gtk_switch_set_active(GTK_SWITCH(switch_to_set), reverse_sense ? !found : found);
+    g_signal_handlers_unblock_by_func(signal_data->mute_switch,
+                                      callback_to_block,
+                                      signal_data);
+    if (label_to_sensitize != NULL) {
+        gtk_widget_set_sensitive(label_to_sensitize, reverse_sense ? found : !found);
+    }
 }
 
 static void
@@ -375,6 +480,319 @@ listbox_remove_all (GtkWidget *widget, gpointer user_data)
     gtk_container_remove (GTK_CONTAINER (container), widget);
 }
 
+static gint
+xfce_notify_sort_apps_in_log (gconstpointer a, gconstpointer b)
+{
+    const LogAppCount *entry1 = *((LogAppCount **) a);
+    const LogAppCount *entry2 = *((LogAppCount **) b);
+
+    gint count_diff = entry2->count - entry1->count;
+    if (count_diff != 0) {
+        return count_diff;
+    } else {
+        return g_utf8_collate(entry1->app_name, entry2->app_name);
+    }
+}
+
+static GPtrArray *
+xfce_notify_count_apps_in_log (GKeyFile *notify_log,
+                               GPtrArray *known_applications)
+{
+    GPtrArray *log_stats = g_ptr_array_new();
+
+    if (known_applications != NULL) {
+        GHashTable *counts = g_hash_table_new(g_str_hash, g_str_equal);
+        GList *keys;
+
+        for (guint i = 0; i < known_applications->len; ++i) {
+            GValue *known_application = g_ptr_array_index(known_applications, i);
+            if (G_VALUE_HOLDS_STRING(known_application)) {
+                g_hash_table_insert(counts, (gchar *)g_value_get_string(known_application), GUINT_TO_POINTER(0));
+            }
+        }
+
+        if (notify_log != NULL) {
+            gsize num_groups = 0;
+            gchar **groups = g_key_file_get_groups(notify_log, &num_groups);
+
+            for (gsize i = 0; i < num_groups; ++i) {
+                gchar *app_name = g_key_file_get_string(notify_log, groups[i], "app_name", NULL);
+
+                if (g_hash_table_contains(counts, app_name)) {
+                    g_hash_table_replace(counts, app_name, GUINT_TO_POINTER(GPOINTER_TO_UINT(g_hash_table_lookup(counts, app_name)) + 1));
+                }
+            }
+        }
+
+        keys = g_hash_table_get_keys(counts);
+        for (GList *l = keys; l != NULL; l = l->next) {
+            LogAppCount *entry = g_new0(LogAppCount, 1);
+            const gchar *app_id = l->data;
+            gchar *app_name;
+
+            app_name = notify_get_from_desktop_file(app_id, G_KEY_FILE_DESKTOP_KEY_NAME);
+            if (app_name == NULL) {
+                /* Fallback: Just set the name provided by the application */
+                app_name = g_strdup(app_id);
+            }
+
+            entry->app_id = app_id;
+            entry->app_name = app_name;
+            entry->count = GPOINTER_TO_UINT(g_hash_table_lookup(counts, app_id));
+            g_ptr_array_add(log_stats, entry);
+        }
+        g_list_free(keys);
+
+        g_hash_table_destroy(counts);
+        g_ptr_array_sort(log_stats, xfce_notify_sort_apps_in_log);
+    }
+
+    return log_stats;
+}
+
+static void
+known_application_signal_data_free(KnownApplicationSignalData *signal_data)
+{
+    g_signal_handlers_disconnect_by_func(signal_data->channel,
+                                         handle_app_switch_property_changed,
+                                         signal_data);
+
+    g_object_unref(signal_data->channel);
+    g_free(signal_data->known_application);
+    g_free(signal_data->known_application_name);
+    g_slice_free(KnownApplicationSignalData, signal_data);
+}
+
+static void
+known_application_delete_clicked(GtkWidget *button,
+                                 KnownApplicationSignalData *signal_data)
+{
+    gchar *confirm_text = g_strdup_printf( _("Are you sure you want to delete notification settings for application \"%s\"?"),
+                                          signal_data->known_application_name);
+    if (xfce_dialog_confirm(GTK_WINDOW(gtk_widget_get_toplevel(button)),
+                            "edit-delete",
+                            _("Delete"),
+                            confirm_text,
+                            _("Forget Application")))
+    {
+        gchar **known_applications = xfconf_channel_get_string_list(signal_data->channel, KNOWN_APPLICATIONS_PROP);
+
+        if (G_LIKELY(known_applications != NULL)) {
+            gchar **muted_applications = xfconf_channel_get_string_list(signal_data->channel, MUTED_APPLICATIONS_PROP);
+            gchar **denied_critical_notifications = xfconf_channel_get_string_list(signal_data->channel, DENIED_CRITICAL_NOTIFICATIONS_PROP);
+
+            xfce_remove_from_string_list_property(signal_data->channel, KNOWN_APPLICATIONS_PROP, known_applications, signal_data->known_application);
+            xfce_remove_from_string_list_property(signal_data->channel, MUTED_APPLICATIONS_PROP, muted_applications, signal_data->known_application);
+            xfce_remove_from_string_list_property(signal_data->channel, DENIED_CRITICAL_NOTIFICATIONS_PROP, denied_critical_notifications, signal_data->known_application);
+
+            g_strfreev(known_applications);
+            g_strfreev(muted_applications);
+            g_strfreev(denied_critical_notifications);
+        }
+    }
+
+    g_free(confirm_text);
+}
+
+static gboolean
+test_gicon_exists(GIcon *gicon,
+                  gint size,
+                  gint scale)
+{
+    GtkIconInfo *icon_info = gtk_icon_theme_lookup_by_gicon_for_scale(gtk_icon_theme_get_default(),
+                                                                      gicon,
+                                                                      size,
+                                                                      scale,
+                                                                      GTK_ICON_LOOKUP_USE_BUILTIN);
+    gboolean exists = icon_info != NULL;
+
+    if (icon_info != NULL) {
+        g_object_unref(icon_info);
+    }
+
+    return exists;
+}
+
+static void
+xfce4_notifyd_known_application_insert_row (XfconfChannel *channel,
+                                            GtkWidget *known_applications_listbox,
+                                            GPtrArray *muted_applications,
+                                            const gchar *const *denied_critical_applications,
+                                            const gchar *const *excluded_from_log_applications,
+                                            const gchar *known_application,
+                                            const gchar *app_name,
+                                            gint count)
+{
+    GtkBuilder *builder;
+    GtkWidget *row;
+    GtkWidget *expander;
+    GtkWidget *app_label;
+    GtkWidget *log_count_label;
+    GtkWidget *icon;
+    GtkWidget *mute_switch;
+    GtkWidget *allow_criticals;
+    GtkWidget *include_in_log;
+    GtkWidget *delete_button;
+    guint i;
+    gint icon_width, icon_height, icon_size;
+    gint scale_factor = gtk_widget_get_scale_factor(known_applications_listbox);
+    gchar *desktop_icon_name = NULL;
+
+    gtk_icon_size_lookup(GTK_ICON_SIZE_LARGE_TOOLBAR, &icon_width, &icon_height);
+    icon_size = MIN(icon_width, icon_height);
+
+    builder = gtk_builder_new_from_string(xfce4_notifyd_config_known_app_ui, xfce4_notifyd_config_known_app_ui_length);
+    if (G_UNLIKELY(builder == NULL)) {
+        g_critical("Unable to load known app UI description");
+        return;
+    }
+
+    expander = GTK_WIDGET(gtk_builder_get_object(builder, "app_item"));
+    icon = GTK_WIDGET(gtk_builder_get_object(builder, "app_icon"));
+    app_label = GTK_WIDGET(gtk_builder_get_object(builder, "app_name"));
+    log_count_label = GTK_WIDGET(gtk_builder_get_object(builder, "app_log_count"));
+    mute_switch = GTK_WIDGET(gtk_builder_get_object(builder, "app_mute"));
+    allow_criticals = GTK_WIDGET(gtk_builder_get_object(builder, "allow_critical_notifications"));
+    include_in_log = GTK_WIDGET(gtk_builder_get_object(builder, "include_in_log"));
+    delete_button = GTK_WIDGET(gtk_builder_get_object(builder, "delete_app"));
+
+    row = gtk_list_box_row_new();
+    gtk_list_box_insert(GTK_LIST_BOX(known_applications_listbox), row, -1);
+    gtk_container_add(GTK_CONTAINER(row), expander);
+
+    /* Number of notifications in the log (if enabled) */
+    if (count > 0) {
+        gtk_label_set_text (GTK_LABEL(log_count_label), g_strdup_printf("%d", count));
+    }
+
+    /* All applications that don't supply their name at all */
+    if (xfce_str_is_empty (known_application)) {
+        const char *format = "<span style=\"italic\">\%s</span>";
+        char *markup;
+
+        known_application = g_strdup(_("Unspecified applications"));
+        markup = g_markup_printf_escaped (format, known_application);
+        gtk_label_set_markup (GTK_LABEL (app_label), markup);
+        g_free (markup);
+
+        gtk_widget_set_sensitive(mute_switch, FALSE);
+        gtk_widget_set_sensitive(allow_criticals, FALSE);
+        gtk_widget_set_sensitive(include_in_log, FALSE);
+        gtk_widget_set_sensitive(delete_button, FALSE);
+    } else {
+        KnownApplicationSignalData *signal_data = g_slice_new0(KnownApplicationSignalData);
+        GIcon *gicon = NULL;
+
+        signal_data->channel = g_object_ref(channel);
+        signal_data->known_application = g_strdup(known_application);
+        signal_data->app_label = app_label;
+        signal_data->mute_switch = mute_switch;
+        signal_data->allow_criticals = allow_criticals;
+        signal_data->include_in_log = include_in_log;
+
+        /* Try to find the correct icon based on the desktop file */
+        desktop_icon_name = notify_get_from_desktop_file (known_application, G_KEY_FILE_DESKTOP_KEY_ICON);
+        if (desktop_icon_name != NULL) {
+            if (g_path_is_absolute(desktop_icon_name)
+                && g_file_test(desktop_icon_name, G_FILE_TEST_EXISTS)
+                && !g_file_test(desktop_icon_name, G_FILE_TEST_IS_DIR))
+            {
+                GFile *file = g_file_new_for_path(desktop_icon_name);
+                gicon = g_file_icon_new(file);
+                g_object_unref(file);
+            } else {
+                gicon = g_themed_icon_new_with_default_fallbacks(desktop_icon_name);
+            }
+
+            if (!test_gicon_exists(gicon, icon_size, scale_factor)) {
+                g_clear_object(&gicon);
+            }
+        }
+
+        /* Fallback: Try to naively load icon names */
+        if (gicon == NULL) {
+            /* Find icons in the right priority:
+               1. normal icon name with fallback
+               2. lowercase icon name with fallback */
+
+            gicon = g_themed_icon_new_with_default_fallbacks(known_application);
+            if (!test_gicon_exists(gicon, icon_size, scale_factor)) {
+                g_clear_object(&gicon);
+            }
+
+            if (gicon == NULL) {
+                gchar *icon_name_dashed = g_strdelimit(g_strdup(known_application), " .", '-');
+
+                gicon = g_themed_icon_new_with_default_fallbacks(icon_name_dashed);
+                if (!test_gicon_exists(gicon, icon_size, scale_factor)) {
+                    gchar *icon_name_dashed_lower = g_ascii_strdown(icon_name_dashed, -1);
+
+                    g_clear_object(&gicon);
+                    gicon = g_themed_icon_new_with_default_fallbacks(icon_name_dashed_lower);
+                    if (!test_gicon_exists(gicon, icon_size, scale_factor)) {
+                        g_clear_object(&gicon);
+                    }
+
+                    g_free(icon_name_dashed_lower);
+                }
+
+                g_free(icon_name_dashed);
+            }
+        }
+
+        if (G_LIKELY(gicon != NULL)) {
+            gtk_image_set_pixel_size(GTK_IMAGE(icon), icon_size);
+            gtk_image_set_from_gicon(GTK_IMAGE(icon), gicon, GTK_ICON_SIZE_LARGE_TOOLBAR);
+            g_object_unref(gicon);
+        }
+
+        signal_data->known_application_name = g_strdup(app_name);
+        gtk_label_set_text(GTK_LABEL(app_label), app_name);
+
+        /* Set correct initial value as to whether an application is muted */
+        if (muted_applications != NULL) {
+            for (i = 0; i < muted_applications->len; i++) {
+                GValue *muted_application;
+                muted_application = g_ptr_array_index (muted_applications, i);
+                if (g_str_match_string (g_value_get_string (muted_application), known_application, FALSE) == TRUE) {
+                    gtk_widget_set_sensitive(app_label, FALSE);
+                    gtk_switch_set_active (GTK_SWITCH (mute_switch), TRUE);
+                    break;
+                }
+            }
+        }
+        g_signal_connect(mute_switch, "state-set",
+                         G_CALLBACK(xfce4_notifyd_mute_switch_activated), signal_data);
+        g_signal_connect(channel, "property-changed::" MUTED_APPLICATIONS_PROP,
+                         G_CALLBACK(handle_app_switch_property_changed), signal_data);
+
+        gtk_switch_set_active(GTK_SWITCH(allow_criticals),
+                              denied_critical_applications == NULL
+                              || !g_strv_contains(denied_critical_applications, known_application));
+        g_signal_connect(allow_criticals, "state-set",
+                         G_CALLBACK(xfce4_notifyd_allow_criticals_switch_activated), signal_data);
+        g_signal_connect(channel, "property-changed::" DENIED_CRITICAL_NOTIFICATIONS_PROP,
+                         G_CALLBACK(handle_app_switch_property_changed), signal_data);
+
+        gtk_switch_set_active(GTK_SWITCH(include_in_log),
+                              excluded_from_log_applications == NULL
+                              || !g_strv_contains(excluded_from_log_applications, known_application));
+        g_signal_connect(include_in_log, "state-set",
+                         G_CALLBACK(xfce4_notifyd_include_in_log_switch_activated), signal_data);
+        g_signal_connect(channel, "property-changed::" EXCLUDED_FROM_LOG_APPLICATIONS_PROP,
+                         G_CALLBACK(handle_app_switch_property_changed), signal_data);
+
+        g_signal_connect(delete_button, "clicked",
+                         G_CALLBACK(known_application_delete_clicked), signal_data);
+
+        g_object_set_data_full(G_OBJECT(row),
+                               "xfce4-notifyd-known-application-signal-data", signal_data,
+                               (GDestroyNotify)known_application_signal_data_free);
+    }
+
+    g_free(desktop_icon_name);
+}
+
 static void
 xfce4_notifyd_known_applications_changed (XfconfChannel *channel,
                                const gchar *property,
@@ -382,19 +800,19 @@ xfce4_notifyd_known_applications_changed (XfconfChannel *channel,
                                gpointer user_data)
 {
     GtkWidget *known_applications_listbox = user_data;
-    GtkWidget *hbox;
-    GtkWidget *label;
-    GtkWidget *icon;
-    GtkWidget *mute_switch;
     GtkCallback func = listbox_remove_all;
     GPtrArray *known_applications;
+    GPtrArray *known_applications_sorted;
     GPtrArray *muted_applications;
-    GValue *known_application;
-    guint i, j;
-    const gchar *icon_name;
+    gchar **denied_critical_applications;
+    gchar **excluded_from_log_applications;
+    GKeyFile *notify_log;
+    guint i;
 
     known_applications = xfconf_channel_get_arrayv (channel, KNOWN_APPLICATIONS_PROP);
     muted_applications = xfconf_channel_get_arrayv (channel, MUTED_APPLICATIONS_PROP);
+    denied_critical_applications = xfconf_channel_get_string_list(channel, DENIED_CRITICAL_NOTIFICATIONS_PROP);
+    excluded_from_log_applications = xfconf_channel_get_string_list(channel, EXCLUDED_FROM_LOG_APPLICATIONS_PROP);
 
     /* TODO: Check the old list versus the new one and only add/remove rows
              as needed instead instead of cleaning up the whole widget */
@@ -402,73 +820,30 @@ xfce4_notifyd_known_applications_changed (XfconfChannel *channel,
     gtk_container_foreach (GTK_CONTAINER (known_applications_listbox), func, known_applications_listbox);
 
     if (known_applications != NULL) {
-        for (i = 0; i < known_applications->len; i++) {
-            GdkPixbuf *pix = NULL;
-            GtkIconInfo *icon_info = NULL;
-            GtkIconInfo *icon_info_lower = NULL;
-            gchar *desktop_icon_name, *icon_name_lower;
+        /* Sort the apps based on their appearance in the log */
+        notify_log = xfce_notify_log_get();
+        known_applications_sorted = xfce_notify_count_apps_in_log(notify_log, known_applications);
+        g_key_file_unref (notify_log);
 
-            known_application = g_ptr_array_index (known_applications, i);
-            hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-            label = gtk_label_new (g_value_get_string (known_application));
-            /* Make sure spaces are converted to dashes so GTK_ICON_LOOKUP_GENERIC_FALLBACK works as expected */
-            icon_name = g_strdelimit ((gchar *) g_value_get_string (known_application)," ",'-');
-            icon_name_lower = g_ascii_strdown (icon_name, -1);
-            icon_info = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default(), icon_name, 24, GTK_ICON_LOOKUP_GENERIC_FALLBACK);
-            icon_info_lower = gtk_icon_theme_lookup_icon (gtk_icon_theme_get_default(), icon_name_lower, 24, GTK_ICON_LOOKUP_GENERIC_FALLBACK);
-            desktop_icon_name = notify_icon_name_from_desktop_id (icon_name_lower);
-            /* Find icons in the right priority: normal icon name with fallback, lowercase icon name with fallback,
-               Desktop file icon property or empty. */
-            if (icon_info) {
-                pix = gtk_icon_info_load_icon (icon_info, NULL);
-                icon = gtk_image_new_from_pixbuf (gdk_pixbuf_scale_simple (pix, 24, 24, GDK_INTERP_BILINEAR));
-            }
-            else if (icon_info_lower) {
-                pix = gtk_icon_info_load_icon (icon_info_lower, NULL);
-                icon = gtk_image_new_from_pixbuf (gdk_pixbuf_scale_simple (pix, 24, 24, GDK_INTERP_BILINEAR));
-            }
-            else if (desktop_icon_name)
-                icon = gtk_image_new_from_icon_name (desktop_icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
-            else
-                icon = gtk_image_new ();
-            g_free (icon_name_lower);
-            g_free (desktop_icon_name);
-            desktop_icon_name = NULL;
-            if (pix)
-                g_object_unref (G_OBJECT (pix));
-            gtk_image_set_pixel_size (GTK_IMAGE (icon), 24);
-
-#if GTK_CHECK_VERSION (3, 16, 0)
-            gtk_label_set_xalign (GTK_LABEL (label), 0);
-#else
-            gtk_widget_set_halign (label, GTK_ALIGN_START);
-#endif
-            mute_switch = gtk_switch_new ();
-            gtk_widget_set_valign (label, GTK_ALIGN_CENTER);
-            gtk_widget_set_valign (mute_switch, GTK_ALIGN_CENTER);
-            gtk_switch_set_active (GTK_SWITCH (mute_switch), TRUE);
-            gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, TRUE, 3);
-            gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 3);
-            gtk_box_pack_end (GTK_BOX (hbox), mute_switch, FALSE, TRUE, 3);
-            gtk_list_box_insert (GTK_LIST_BOX (known_applications_listbox), hbox, -1);
-            /* Set correct initial value as to whether an application is muted */
-            if (muted_applications != NULL) {
-                for (j = 0; j < muted_applications->len; j++) {
-                    GValue *muted_application;
-                    muted_application = g_ptr_array_index (muted_applications, j);
-                    if (g_str_match_string (g_value_get_string (muted_application), g_value_get_string (known_application), FALSE) == TRUE) {
-                        gtk_switch_set_active (GTK_SWITCH (mute_switch), FALSE);
-                        break;
-                    }
-                    else
-                        continue;
-                }
-            }
-            g_signal_connect (G_OBJECT (mute_switch), "state-set", G_CALLBACK (xfce4_notifyd_switch_activated), channel);
+        for (i = 0; i < known_applications_sorted->len; i++) {
+            LogAppCount *application = g_ptr_array_index(known_applications_sorted, i);
+            xfce4_notifyd_known_application_insert_row(channel,
+                                                       known_applications_listbox,
+                                                       muted_applications,
+                                                       (const gchar *const *)denied_critical_applications,
+                                                       (const gchar *const *)excluded_from_log_applications,
+                                                       application->app_id,
+                                                       application->app_name,
+                                                       application->count);
+            g_free(application->app_name);
         }
+        g_ptr_array_free(known_applications_sorted, TRUE);
     }
+
     xfconf_array_free (known_applications);
     xfconf_array_free (muted_applications);
+    g_strfreev(denied_critical_applications);
+    g_strfreev(excluded_from_log_applications);
     gtk_widget_show_all (known_applications_listbox);
 }
 
@@ -545,6 +920,7 @@ xfce4_notifyd_log_populate (NotificationLogWidgets *log_widgets)
     GdkPixbuf *pixbuf = NULL;
     gchar *notify_log_icon_folder;
     gchar *notify_log_icon_path;
+    gint scale_factor = gtk_widget_get_scale_factor (log_listbox);
 
     today = g_date_time_new_now_local ();
     timestamp = g_date_time_format (today, "%F");
@@ -605,39 +981,38 @@ xfce4_notifyd_log_populate (NotificationLogWidgets *log_widgets)
             markup = g_markup_printf_escaped (format, tmp);
             g_free (tmp);
             summary = gtk_label_new (NULL);
-            gtk_label_set_markup (GTK_LABEL (summary), markup);
-#if GTK_CHECK_VERSION (3, 16, 0)
+            if (xfce_notify_is_markup_valid(markup)) {
+                gtk_label_set_markup (GTK_LABEL (summary), markup);
+            } else {
+                gtk_label_set_text(GTK_LABEL(summary), markup);
+            }
             gtk_label_set_xalign (GTK_LABEL (summary), 0);
-#else
-            gtk_widget_set_halign (summary, GTK_ALIGN_START);
-#endif
             g_free (markup);
             tmp = g_key_file_get_string (notify_log, group, "body", NULL);
             body = gtk_label_new (NULL);
             g_object_ref (body);
-            gtk_label_set_markup (GTK_LABEL (body), tmp);
-            if (g_strcmp0 (gtk_label_get_text (GTK_LABEL (body)), "") == 0) {
-                gchar *tmp1;
-
-                tmp1 = g_markup_escape_text (tmp, -1);
-                gtk_label_set_text (GTK_LABEL (body), tmp1);
-                g_free (tmp1);
+            if (xfce_notify_is_markup_valid(tmp)) {
+                gtk_label_set_markup (GTK_LABEL (body), tmp);
+            } else {
+                gtk_label_set_text(GTK_LABEL(body), tmp);
             }
             g_free (tmp);
-#if GTK_CHECK_VERSION (3, 16, 0)
             gtk_label_set_xalign (GTK_LABEL (body), 0);
-#else
-            gtk_widget_set_halign (body, GTK_ALIGN_START);
-#endif
             gtk_label_set_ellipsize (GTK_LABEL (body), PANGO_ELLIPSIZE_END);
             tmp = g_key_file_get_string (notify_log, group, "app_icon", NULL);
             notify_log_icon_path = g_strconcat (notify_log_icon_folder , tmp, ".png", NULL);
             if (g_file_test (notify_log_icon_path, G_FILE_TEST_EXISTS)) {
                 pixbuf = gdk_pixbuf_new_from_file_at_scale (notify_log_icon_path,
-                                                            24, 24, FALSE, NULL);
-                app_icon = gtk_image_new_from_pixbuf (pixbuf);
-                if (pixbuf)
+                                                            24 * scale_factor, 24 * scale_factor,
+                                                            TRUE, NULL);
+                if (G_LIKELY (pixbuf != NULL)) {
+                    cairo_surface_t *surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale_factor, NULL);
+                    app_icon = gtk_image_new_from_surface (surface);
                     g_object_unref (pixbuf);
+                    cairo_surface_destroy (surface);
+                } else {
+                    app_icon = gtk_image_new ();
+                }
             } else {
                 app_icon = gtk_image_new_from_icon_name (tmp, GTK_ICON_SIZE_LARGE_TOOLBAR);
                 gtk_image_set_pixel_size (GTK_IMAGE (app_icon), 24);
@@ -703,6 +1078,8 @@ xfce4_notifyd_log_populate (NotificationLogWidgets *log_widgets)
     g_free (notify_log_icon_folder);
     g_date_time_unref (today);
     g_free (timestamp);
+    if (notify_log)
+        g_key_file_unref (notify_log);
 
     gtk_widget_show_all (log_listbox);
 
@@ -793,7 +1170,9 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
     GtkToolItem *log_open_button;
     GtkWidget *icon;
     GtkWidget *primary_monitor;
+    GtkWidget *mute_sounds;
     GtkWidget *do_fadeout;
+    GtkWidget *show_text_with_gauge;
     GtkAdjustment *adj;
     GError *error = NULL;
     gchar *current_theme;
@@ -846,6 +1225,15 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
     if(gtk_combo_box_get_active(GTK_COMBO_BOX(primary_monitor)) == -1)
         gtk_combo_box_set_active(GTK_COMBO_BOX(primary_monitor), 0);
 
+    mute_sounds = GTK_WIDGET(gtk_builder_get_object(builder, "mute_sounds"));
+#ifdef ENABLE_SOUND
+    xfconf_g_property_bind(channel, MUTE_SOUNDS_PROP, G_TYPE_BOOLEAN,
+                           G_OBJECT(mute_sounds), "active");
+#else
+    gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(builder, "mute_sounds_label")));
+    gtk_widget_hide(mute_sounds);
+#endif
+
     // Appearance
     theme_combo = GTK_WIDGET(gtk_builder_get_object(builder, "theme_combo"));
     current_theme = xfconf_channel_get_string(channel, "/theme", "Default");
@@ -891,6 +1279,10 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
         gtk_widget_set_sensitive (slideout_widgets.do_slideout, FALSE);
     }
 
+    show_text_with_gauge = GTK_WIDGET(gtk_builder_get_object(builder, "show_text_with_gauge"));
+    xfconf_g_property_bind(channel, "/show-text-with-gauge", G_TYPE_BOOLEAN,
+                           G_OBJECT(show_text_with_gauge), "active");
+
     btn = GTK_WIDGET(gtk_builder_get_object(builder, "preview_button"));
     g_signal_connect(G_OBJECT(btn), "clicked",
                      G_CALLBACK(xfce_notifyd_config_preview_clicked), NULL);
@@ -900,6 +1292,7 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
      *******************/
     known_applications_scrolled_window = GTK_WIDGET (gtk_builder_get_object (builder, "known_applications_scrolled_window"));
     known_applications_listbox = gtk_list_box_new ();
+    gtk_list_box_set_selection_mode(GTK_LIST_BOX(known_applications_listbox), GTK_SELECTION_NONE);
     gtk_container_add (GTK_CONTAINER (known_applications_scrolled_window), known_applications_listbox);
     gtk_list_box_set_header_func (GTK_LIST_BOX (known_applications_listbox), display_header_func, NULL, NULL);
 
@@ -910,9 +1303,6 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
     xfce4_notifyd_known_applications_changed (channel, KNOWN_APPLICATIONS_PROP, NULL, known_applications_listbox);
     gtk_list_box_set_placeholder (GTK_LIST_BOX (known_applications_listbox), placeholder_label);
     gtk_widget_show_all (placeholder_label);
-    g_signal_connect (G_OBJECT (known_applications_listbox), "row-activated",
-                      G_CALLBACK (xfce4_notifyd_row_activated),
-                      channel);
     g_signal_connect (G_OBJECT (channel),
                       "property-changed::" KNOWN_APPLICATIONS_PROP,
                       G_CALLBACK (xfce4_notifyd_known_applications_changed), known_applications_listbox);
